/*
	-- SurveryResult ASP.NET Server Control --
	-- Copyright (C) 2004. All rights reserved. --
	
	Author: Stephen Stchur	
	Created: 06.08.04
	
	PURPOSE:
	The purpose of this control is to provide a simple, yet powerful mechanism of displaying
	the results that are returned from a WebSurvey Control.
	
	CONCEPT:
	The SurveyResult control expects the following properties:
	SurveyFile		- relative path to the XML file which contains the survey of interest
	AnswersFile		- relative path to SurveyFile's corresponding answers (in XML format of course)
	Stylesheet		- stylesheet (XSL file) which will generate the visual representation of the control
	XQuery1				- comma separated list of XQueries that will be executed against the SurveyFile
	XQuery2				- comma separated list of XQueries that will be executed against the AnswersFile
	
	The control works by first reading the SurveyFile and the AnswersFile into memory.  It then
	executes all of the XQueries from XQuery1 on the SurveyFile and stores the results in an
	XmlNodeList array.
	
	Second, the control will execute all of the XQueries from XQuery2 on the AnswersFile, and those
	results will be stored in a second XmlNodeList array.
	
	Third, a new XmlDocument object will be created, whose root node is <SurveyResult>.  All of the
	nodes from XQuery1's XmlNodeList array and all of the nodes from XQuery2's XmlNodeList array
	will be "merged" together into the new XmlDocument (as children of the <SurveyResult> node).
	
	Finally, in the Render() method, an XslTranform object will be created and loaded with an
	XSL stylesheet (the one provided from the Stylesheet property).  The Transform() method will
	then be called, which will transform the XmlDocument created in part three into XHTML, based
	on the specifications from the XSL stylesheet.  The results of the transformation will be
	written to a StringWriter object.
	
	Once the StringWriter object contains the XHTML representing the SurveyResults, it will be
	send to the HtmlTextWriter object and rendered to the screen.
	
	NOTES:
	The SurveyResult control does not check to make sure that there is a logical connection between
	the SurveyFile and the Answersfile -- this is left to the user.
	
	All properties are required.  No error trapping is provided for missing properties.	
*/

using System;																					// EventArgs
using System.IO;																			// StringWriter
using System.Web;																			// HttpContext
using System.Web.UI;																	// HtmlTextWriter
using System.Xml;																			// XmlDocument, XmlNode, XmlElement, etc...
using System.Xml.Xsl;																	// XslTransform

namespace sstchur
{
	namespace web
	{		
		namespace survey
		{
			[ System.ComponentModel.ToolboxItem(false) ]
			public class SurveyResult : System.Web.UI.WebControls.WebControl
			{
				// Privates Member Variables
				private string m_strSurveyFile = "";
				private string m_strAnswersFile = "";
				private string m_strStylesheet = "";
				private string m_strXQuery1 = "";
				private string m_strXQuery2 = "";			
				private XmlDocument m_docResults = new XmlDocument();
				
				// Public Properties
				
				// SurveyFile : required
				// Relative path to the XML file containing the survey of interest
				public string SurveyFile
				{
					set { m_strSurveyFile = HttpContext.Current.Server.MapPath(value); }
				}
				
				// AnswersFile : required
				// Relative path to the XML file containing the answers corresponding to SurveyFile
				public string AnswersFile
				{
					set { m_strAnswersFile = HttpContext.Current.Server.MapPath(value); }
				}
				
				// Stylesheet : required
				// Relative path to the XSL file used to create the visual representation of the control
				public string Stylesheet
				{
					set { m_strStylesheet = HttpContext.Current.Server.MapPath(value); }
				}
				
				// XQuery1 : required
				// Comma separated list of XQueries that will be executed on SurveyFile
				public string XQuery1
				{
					set { m_strXQuery1 = value; }
				}
				
				// XQuery2 : required
				// Comma separated list of XQueries that will be execured on AnswersFile
				public string XQuery2
				{
					set { m_strXQuery2 = value; }
				}
				
				// Constructor
				public SurveyResult()		
				{
					/* m_docResults is the XmlDocument object that will hold the resuls of the "merge" of
						the SurveyFile and the AnswersFile. */
					
					// Create an XML Declaration and insert it into the m_docResults XmlDocument
					XmlDeclaration xmlDecl = m_docResults.CreateXmlDeclaration("1.0", null, null);
					m_docResults.AppendChild(xmlDecl);
					
					// Create the root node <SurveyResult> and insert it into the m_docResults XmlDocument
					XmlElement root = m_docResults.CreateElement("SurveyResult");
					m_docResults.AppendChild(root);
				}
				
				// On Init -- before the control renders
				protected override void OnInit(EventArgs e)
				{
					// Load the SurveyFile into memory
					XmlDocument docSurvey = new XmlDocument();
					docSurvey.Load(m_strSurveyFile);
					
					// Load the AnswersFile into memory
					XmlDocument docAnswers = new XmlDocument();
					docAnswers.Load(m_strAnswersFile);				
					
					// Get the XmlNodeList array that was generated from the XQueries in XQuery1
					// and append the resulting Nodes to the m_docResults XmlDocument
					XmlNodeList[] nlSurvey = GetNodes(docSurvey, m_strXQuery1);
					AppendNodeList(m_docResults, nlSurvey);
					
					// Get the XmlNodeList array that was generated from the XQueryes in XQuery2
					// and append the resulting Nodes to the m_docResults XmlDocument
					XmlNodeList[] nlAnswers = GetNodes(docAnswers, m_strXQuery2);
					AppendNodeList(m_docResults, nlAnswers);
				}
				
				// Render -- when the control is ready to render itself to the page
				protected override void Render(HtmlTextWriter htmlWriter)
				{				
					// Create an XslTransform object and load into it, the stylesheet provided in
					// the Stylesheet property
					XslTransform xslt = new XslTransform();
					xslt.Load(m_strStylesheet);
					
					// Create a StringWriter object to hold the results of the XSL transformation
					// and call the Transform method on the m_docResults XmlDocument
					StringWriter strWriter = new StringWriter();
					xslt.Transform(m_docResults, null, strWriter, null);
					
					// Render the results to the page
					htmlWriter.Write(strWriter.ToString());					
				}
				
				
				/* Private member functions */
				
				// AppendNodeList
				// Adds all the nodes in _nodeListArray to the root of the XmlDocument, _doc			
				private void AppendNodeList(XmlDocument _doc, XmlNodeList[] _nodeListArray)
				{	
					foreach (XmlNodeList list in _nodeListArray)
					{
						foreach (XmlNode node in list)
						{
							_doc.DocumentElement.AppendChild(_doc.ImportNode(node, true));						
						}
					}
				}
				
				// GetNodes
				// Returns an array of XmlNodeLists, based on the querys provided in _queryList 
				private XmlNodeList[] GetNodes(XmlDocument _doc, string _queryList)
				{
					// Create an array of XQuery expressions
					string[] xQuery = _queryList.Split(new char[] { ',' });
					
					// Create a pointer to the root of the XmlDocument
					XmlElement root = _doc.DocumentElement;
					
					// Create an array of XmlNodeLists whose length = the number of queries in the xQuery array
					XmlNodeList[] nodeListArray = new XmlNodeList[xQuery.Length];
					
					// Create a counter for the NodeLists, initialize it to zero
					int nlCounter = 0;
					
					// For each XQuery, create a NodeList to store the results
					for (int i = 0; i < xQuery.Length; i++)				
					{
						string xQry = xQuery[i];
						XmlNodeList nodeList = root.SelectNodes(xQry);					
						nodeListArray[nlCounter] = nodeList;
						nlCounter++;					
					}
					
					// return the NodeList array
					return nodeListArray;				
				}
			}	
		}
	}
}